
#include "menu.h"
#include "filedialog.h"
#include "icon.h"
#include "configuration.h"

#include <stdio.h>

#define CMD_QUIT     1
#define CMD_OPEN     2
#define CMD_SAVE     3

#define CMD_ABOUT    9
#define FILE_OPEN    10

class testaxeconf: public configuration
{
};

popup_menu *edit_menu;

struct cmd_map_entry
{
   int   code;
   int   (*handler)(void);
};

class aboutbox : public managed_window
{
   icon            *logo;
   command_button  *ok;

public:

   aboutbox() : managed_window("About AXE")
   {
      Background(Black);
      logo  = new icon(*this, 10, 40, "andromeda.xpm");
      ok = new command_button(*this, 300, 50, "OK", 0);
      Resize(400, 128);
      SelectInput(ExposureMask, 1);
   }

   virtual int EV_Expose(XExposeEvent ev)
   {
      DrawString(white_gc, 110, 30, "Andromeda X Windows Encapsulation");
      DrawString(white_gc, 110, 95, "Version 0.3 - November 05, 2002");
      return 1;
   }

   int  DoCommand(int code)
   {
      Unmap();
      return 1;
   }
};

class xapp: public xapplication
{
   managed_window *main_frame;
   menu_item *save_item;
   file_dialog *FileDialog;
   aboutbox    *About;

   testaxeconf    config;

   virtual void SetupResources(void);

   virtual int DoCommand(int code);

public:

   int OnOpen(void);

   int OnSave(void);

   int OnQuit(void) { printf("OnQuit\n"); return 0; }

};

xapp Application;


int xapp::OnOpen(void)
{
   FileDialog->Realize();
   return 1;
}

int xapp::OnSave(void)
{
   FileDialog->Realize();
   return 1;
}

/*  The pinguin icon as a static array */

#include "pinguin.xpm"

void xapp::SetupResources(void)
{
   window *menu;
   frame  *fr;
   scrollbar  *scr;
   edit *edt;
   menu_item *item;
   popup_menu *popup;

   config.read("testaxe");

   About      = new aboutbox;
   FileDialog = new file_dialog(FILE_OPEN);

   main_frame = new managed_window;
   main_frame->Command_WhenClosed(CMD_QUIT);

   String bgcolor = config.find_parameter("colors", "background");
   if (bgcolor != "")
   {
      main_frame->Background(color(bgcolor));
   }

   fr = new button(*main_frame, 20, 30, "Up");

   fr = new toggle_button(*main_frame, 20, 100, "Down");

   fr = new frame(*main_frame, 180, 30, "Left");
   fr->TextAlign(Left, Middle);

   fr = new frame(*main_frame, 180, 70, "Center");
   fr->TextAlign(Center, Middle);
   fr->Strength(0);

   fr = new frame(*main_frame, 180, 110, "Right");
   fr->TextAlign(Right, Middle);

   fr = new frame(*main_frame, 180, 150, "Top");
   fr->TextAlign(Center, Top);

   fr = new frame(*main_frame, 180, 190, "Middle");
   fr->TextAlign(Center, Middle);
   fr->Strength(0);

   fr = new frame(*main_frame, 180, 230, "Bottom");
   fr->TextAlign(Center, Bottom);
   fr->WindowGravity(SouthEastGravity);

   scr = new scrollbar(*main_frame, 200, 130, 30);
   scr->SetSizes(1000, 200);
   scr->Realize();

   edt = new edit(*main_frame, 20, 140, "This is editable text");
   edt = new edit(*main_frame, 20, 170, "And another text field");

   menu = new menu_bar(*main_frame);

   item = new menu_label(*menu, "Help");
   popup = new popup_menu;
   item->Popup(popup);
   item  = new menu_label(*popup, "About...", CMD_ABOUT);
   item->Map();

   item = new menu_label(*menu, "View");

   popup = new popup_menu;
   item->Popup(popup);

   item = new menu_label(*popup, "Status");
   item->Map();

   item = new menu_label(*popup, "Tools");
   item->Map();

   item = new menu_label(*menu, "Edit");

   popup = new popup_menu;
   item->Popup(popup);
   edit_menu = popup;

   item = new menu_label(*popup, "Paste");
   item->Map();

   item = new menu_label(*popup, "Copy");
   item->Map();

   item = new menu_label(*popup, "Cut");
   item->Map();

   item = new menu_label(*menu, "File");

   popup = new popup_menu;
   item->Popup(popup);   

   item = new menu_label(*popup, "Quit", CMD_QUIT);
   item->Map();

   item = new menu_separator(*popup);
   item->Map();

   save_item = new menu_label(*popup, "Save", CMD_SAVE);
   save_item->Map();
   save_item->Enable(0);

   item = new menu_label(*popup, "Open...", CMD_OPEN);
   item->Map();

   item = new menu_label(*popup, "New ->");
   item->Map();

   popup = new popup_menu;
   item->Popup(popup);

   item = new menu_label(*popup, "Type B");
   item->Map();

   item = new menu_label(*popup, "Type A");
   item->Map();

   icon *logo;
   logo = new icon(*main_frame, 10, 200, pinguin_xpm);
   main_frame->Realize();
   menu->Realize();

   pixmap   pinguin(pinguin_xpm);

   main_frame->DrawPixmap(text_normal_gc, 270, 100, pinguin);
}

int xapp::DoCommand(int code)
{
   printf("Command : %d\n", code);

   switch (code)
   {
   case CMD_QUIT:
      return 0;

   case CMD_OPEN:
      OnOpen();
      return 1;

   case FILE_OPEN:
      std::cout << "Opening " << FileDialog->PathName() << "\n";
      save_item->Enable(1);
      return 1;

   case CMD_SAVE:
      OnSave();
      save_item->Enable(0);
      return 1;

   case CMD_ABOUT:
      About->Realize();
      return 1;

   default:
      return 1;
   }
}
