/*
 *  C++ X Windows class library
 */

#include <X11/Xlib.h>

class drawable;
class window;

class point
{
   short  _x, _y;

public:

   point()   { _x = 0; _y = 0; }

   point(int x, int y)   { _x = x; _y = y; }

   short x()   { return _x; }

   short y()   { return _y; }

};


class line
{
   int   npoints;
   int   mode;
   point *points;

   friend drawable;

   void drawline(drawable &d);

public:

   line()
   {
      npoints = 0;
      mode = CoordModeOrigin;
   }

   line(short x1, short y1, short x2, short y2)
   {
      npoints = 2;
      mode = CoordModeOrigin;
      points = new point[2];
      points[0] = point(x1, y1);
      points[1] = point(x2, y2);
   }

   line(point p1, point p2)
   {
      npoints = 2;
      mode = CoordModeOrigin;
      points = new point[2];
      points[0] = p1;
      points[1] = p2;
   }

   ~line()
   {
      if (npoints != 0)
         delete [] points;
   }

   point &operator[] (int index);

};

class rectangle
{
   short          x, y;
   unsigned short width, height;

   friend drawable;

public:

   rectangle()
   {
      x      = 0;
      y      = 0;
      width  = 0;
      height = 0;
   }

   rectangle(int _x, int _y, unsigned int _w, unsigned int _h)
   {
      x      = _x;
      y      = _y;
      width  = _w;
      height = _h;
   }

   //  Define a rectangle by its diagonal

   rectangle(point p1, point p2);

//   friend window &operator<< (window &d, rectangle &r);
//   friend pixmap &operator<< (pixmap &d, rectangle &r);
};

class box
{
   short          x, y;
   unsigned short width, height;

public:

   box()
   {
      x      = 0;
      y      = 0;
      width  = 0;
      height = 0;
   }

   box(int _x, int _y, unsigned int _w, unsigned int _h)
   {
      x      = _x;
      y      = _y;
      width  = _w;
      height = _h;
   }

   //  Define a box by its diagonal

   box(point p1, point p2);

//   friend window &operator<< (window &d, box &b);
//   friend pixmap &operator<< (pixmap &d, box &b);
};

class text
{
   short x, y;
   String txt;

   friend drawable;

public:

   text()
   {
      x = 0;
      y = 0;
   }

   text(int _x, int _y, String str)
   {
      x = _x;
      y = _y;
      txt = str;
   }

   text(point p, String s)
   {
      x = p.x();
      y = p.y();
      txt = s;
   }

//   friend window &operator<< (window &d, text &t);
//   friend pixmap &operator<< (pixmap &d, text &t);
};

class drawable
{
   friend window;

   XID   id;   // The window or pixmap id in the server.
   GC    gc;   // The graphics context.

public:

   XID  Id()
   {
      return id;
   }

   GC  Gc()
   {
      return gc;
   }

   void foreground(unsigned long color)
   {
      XSetForeground(stddpy.Dpy(), gc, color);
   }

   void background(unsigned long color)
   {
      XSetBackground(stddpy.Dpy(), gc, color);
   }

   drawable &operator<< (point &p);
   drawable &operator<< (line &l);

   drawable &operator<< (rectangle &r);
   drawable &operator<< (text &t);
};

class window : public drawable
{
public:
   window(int x=0, int y=0, unsigned int w=100, unsigned int h=100);
   window(window &, int x=0, int y=0, unsigned int w=100, unsigned int h=100);
   ~window();

   //   Mapping and unmapping

   void operator++()
   {
      XMapWindow(stddpy.Dpy(), id);
   }
   
   void operator--()
   {
      XUnmapWindow(stddpy.Dpy(), id);
   }
   
};

class pixmap : public drawable
{
};

