/**************************************************************************
**  (c) Copyright 2002, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : configuration.h
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.4 $
**
**  DESCRIPTION      : Definition of configuration class 
**
**  EXPORTED OBJECTS : class configuration
**  LOCAL    OBJECTS : 
**  MODULES  USED    : String
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Nov 02, 2002
**      LAST UPDATE     : 
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: configuration.h,v $
   Revision 1.4  2007/05/04 14:01:22  arjen
   Added new arguments to configuration::find_parameter() to allow for a list of
   sections and a list of parameters in sections.
   The new arguments are section index and parameter index (default = 0).

   Revision 1.3  2002/11/02 14:35:15  arjen
   Uses the XML2 library
   BugFix: segfault when finding a parameters that is not in the user-sepcific
   config tree.

   Revision 1.2  2002/09/02 06:18:20  arjen
   Fixed some date and time conversion functions

   Revision 1.1  2002/07/25 08:01:26  arjen
   First checkin, AXE release 0.2

*****************************/

/* static const char *RCSID = "$Id: configuration.h,v 1.4 2007/05/04 14:01:22 arjen Exp $"; */

#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include <libxml/parser.h>   // usually in /usr/include/libxml2, see xml2-config

#include "String.h"

/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : configuration
//  BASECLASS      :
//  MEMBERS        : 
//  OPERATORS      :
//  METHODS        : read
//                   find_parameter
//
//  DESCRIPTION    : Handle configurational parameters for the application.
//                   Many applications need some permanently stored configurational
//                   data. The information is usually stored in two places: A system-
//                   wide configuration file and a configuration file per user.
//                   The content of the configuration file is in XML format.
//                   The configuration base class takes care of finding the configuration
//                   files, e.g. in /etc/app.conf or in /usr/local/etc/app.conf, along
//                   with the configuration file in the user's home directory.
//                   The config files are parsed with the gnome XML parser and a
//                   framework is provided to find configurational items.
//
//  RELATIONS      : 
//  SEE ALSO       :
//  LAST MODIFIED  : Apr 13, 2007
///////////////////////////////////////////////////////////////////////////
*/

class configuration
{

   xmlDocPtr     system;   // The system-wide config tree
   xmlDocPtr     user;     // User-specific config tree
   String        app_name; // Name of the application and XML root element

   xmlNodePtr    xmlFindTag(xmlNodePtr, const String, int n = 0);

public:

   configuration()
   {
      system  = NULL;
      user    = NULL;
      app_name = "AXE";
   }

   //  Find the config files and parse the XML

   bool read(const String name);

   //  Return the value of a config parameter

   String find_parameter(const String section, const String parameter, int sec_n = 0, int param_n = 0);

};

#endif // CONFIGURATION_H
