/**************************************************************************
**  (c) Copyright 1998, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : display.h
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.2 $
**
**  DESCRIPTION      : Definition of display class 
**
**  EXPORTED OBJECTS : class display
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Feb 06, 1998
**      LAST UPDATE     : Feb 07, 1998
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: display.h,v $
   Revision 1.2  2002/11/04 07:24:31  arjen
   Use proper namespace for iostream classes

   Revision 1.1  2002/07/25 08:01:26  arjen
   First checkin, AXE release 0.2

*****************************/

/* static const char *RCSID = "$Id: display.h,v 1.2 2002/11/04 07:24:31 arjen Exp $"; */

#ifndef _DISPLAY_H
#define _DISPLAY_H

#include <iostream>
#include <X11/Xlib.h>

/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : display
//  BASECLASS      :
//  MEMBERS        : dpy   : Pointer to the Xlib Display
//  OPERATORS      : >> XEvent  - Read next event from the queue
//  METHODS        : display()       - Connect the default display
//                   display(char *) - Connect a specific display.
//                   ~display()      - Close the display connection
//
//                   Root()          - The root window.
//                   Black()         - Black pixel value.
//                   White()         - White pixel value.
//  DESCRIPTION    : A display object maintains the connection to the X
//                   workstation. The library contains one such object, stddpy
//                   which is connected to the default display.
//
//  RELATIONS      :
//  SEE ALSO       :
//  LAST MODIFIED  :
///////////////////////////////////////////////////////////////////////////
*/

class display
{
   Display  *dpy;

   GC       default_gc;

public:

   display()
   {
      Status s;
      s = XInitThreads();
      dpy = XOpenDisplay("");
      default_gc = XCreateGC(dpy, DefaultRootWindow(dpy), 0, 0);
   }

   //  contructor with char* argument opens named display.
   display(char *display_name)
   {
      dpy = XOpenDisplay(display_name);
      default_gc = XCreateGC(dpy, DefaultRootWindow(dpy), 0, 0);
   }

   ~display()
   {
      XFreeGC(dpy, default_gc);
      XCloseDisplay(dpy);
   }

   display & operator>> (XEvent &ev)
   {
      XNextEvent(dpy, &ev);
      return *this;
   }

   //  return display pointer.
   Display *Dpy()
   {
      return dpy;
   }

   operator Display *()
   {
      return dpy;
   }

   Window Root()
   {
      return DefaultRootWindow(dpy);
   }

   //  Obtain the color index value for a white pixel.
   unsigned long White()
   {
      return WhitePixel(dpy, DefaultScreen(dpy));
   }

   //  Obtain the color index value for a black pixel.
   unsigned long Black()
   {
      return BlackPixel(dpy, DefaultScreen(dpy));
   }

   GC  gc()
   {
      return default_gc;
   }

   Colormap ColMap()
   {
      return DefaultColormap(dpy, DefaultScreen(dpy));
   }
};

extern display stddpy;

#endif /* _DISPLAY_H */
