
/**************************************************************************
**  (c) Copyright 1998, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : edit.h
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.1 $
**
**  DESCRIPTION      :  Definition of text editor classes
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Mar 13, 1998
**      LAST UPDATE     : Jun 18, 1998
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: edit.h,v $
   Revision 1.1  2002/07/25 08:01:26  arjen
   First checkin, AXE release 0.2

*****************************/

/* static const char *RCSID = "$Id: edit.h,v 1.1 2002/07/25 08:01:26 arjen Exp $"; */

#ifndef AXE_EDIT_H
#define AXE_EDIT_H

#include "xappl.h"

#include "String.h"

/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : edit
//  BASECLASS      : window
//  MEMBERS        : cursor_position : Index in the text string where edit
//                         operations occur. A visible cursor is drawn before
//                         that character when we have the focus.
//                   selected_until  : If text is selected, this is the index
//                         in the text string that marks the end of the selec-
//                         ted text. Characters are selected from the
//                         cursor_position until here or vice versa.
//  OPERATORS      :
//  METHODS        :
//
//  DESCRIPTION    :
//
//  RELATIONS      :
//  SEE ALSO       :
//  LAST MODIFIED  : Jun 18, 1998
///////////////////////////////////////////////////////////////////////////
*/

class edit : public window
{
   int        selected_until;    //  End of selected text
   int        text_position;     //  Graphical position in window.

   enum       { No_Focus, Implicit_Focus, Explicit_Focus } focus_state;


   int  map_to_textposition(int x);
   int  map_to_pixelposition(int tp);
   void toggle_cursor(void);

protected:

   String     text;
   int        cursor_position;   //  Text cursor in String

   virtual int EV_KeyPress(XKeyEvent ev);
   virtual int EV_ButtonPress(XButtonEvent ev);
   virtual int EV_ButtonRelease(XButtonEvent ev);
   virtual int EV_MotionNotify(XMotionEvent ev);
   virtual int EV_FocusIn(XFocusChangeEvent ev);
   virtual int EV_FocusOut(XFocusChangeEvent ev);
   virtual int EV_EnterNotify(XCrossingEvent ev);
   virtual int EV_LeaveNotify(XCrossingEvent ev);

public:

   edit(window &par, int x, int y, const String &txt)
                 : window(par, x, y, 100, 20)
   {
      text = txt;
      cursor_position = 0;
      selected_until = -1;
      text_position = 2;
      focus_state = No_Focus;
      SelectInput(ExposureMask|KeyPressMask, 1);
      SelectInput(EnterWindowMask|LeaveWindowMask|FocusChangeMask, 1);
      SelectInput(ButtonPressMask|Button1MotionMask|ButtonReleaseMask, 1);
      XDefineCursor(stddpy, ID(), text_cursor);
   }

   edit & operator =(const String &s)
   {
      text = s;
      redraw();

      return *this;
   }

   operator String &()
   {
      return text;
   }

   void redraw(void);

   virtual int EV_Expose(XExposeEvent ev)
   {
      redraw();
      return 1;
   }

   virtual void enter(void);
   virtual void focuslost(void);

   void SelectAll(void);
   int  ClearSelection(void);
   int  DeleteSelection(void);
};

#endif /* AXE_EDIT_H */
