
#include "filedialog.h"

#define OPEN_DLG_OK  100
#define OPEN_DLG_CANCEL 101

#include <algorithm>

#include <stdio.h>

#include "directory.xpm"
#include "regfile.xpm"

bool operator < (const file_entry &fe1, const file_entry &fe2)
{
   if (S_ISDIR(fe1.st.st_mode) == S_ISDIR(fe2.st.st_mode))
   {
      return fe1.name < fe2.name;
   }
   else
   {
      return S_ISDIR(fe1.st.st_mode);
   }
}

directory_view::directory_view(window &parent) : table_view(parent, 200, 2)
{
   width(0) = 20;
   width(1) = 330;
   head(1)  = " Name ";
   Resize(360, 220);
   dir.create(directory_xpm);
   reg.create(regfile_xpm);
   cwd = ".";
}

int directory_view::scan()
{
   DIR           *dirstream;
   struct dirent *entry;
   unsigned       i;

   d_list.clear();
   dirstream = opendir(cwd);
   entry = readdir(dirstream);

   while (entry)
   {
      file_entry  file;
   
      file.name = entry->d_name;
      stat(entry->d_name, &file.st);
      d_list.push_back(file);

      entry = readdir(dirstream);
   }
   closedir(dirstream);

   sort(d_list.begin(), d_list.end());

   recreate(d_list.size());

   for (i = 0; i < d_list.size(); i++)
   {
      cell(i, 1) = d_list[i].name;
   }

   return i;
}

//   Change the view's directory and return the new directory pathname

String directory_view::changedir(String dir)
{
   char   process_dir[PATH_MAX];  //  Remember the process cwd
   char   destination_dir[PATH_MAX];

   getcwd(process_dir, PATH_MAX);

   chdir(cwd);
   if (chdir(dir) == -1)
   {
      int  slash;

      perror("File selector");
      slash = dir.rindex('/');
      dir = dir(0, slash);
      chdir(dir);
   }
   getcwd(destination_dir, PATH_MAX);
   cwd = destination_dir;
   scan();
   redraw();

   chdir(process_dir);           //  Restore the directory we came from

   return cwd;
}

void directory_view::redraw()
{
   int    i;
   int    y;

   table_view::redraw();
   y = 22;
   i = top_row();

   while ((unsigned)i < d_list.size() && (unsigned)y < Size().h)
   {
      if (S_ISDIR(d_list[i].st.st_mode))
      {
         DrawPixmap(text_normal_gc, 2, y, dir);
      }
      else
      {
         DrawPixmap(text_normal_gc, 2, y, reg);
      }
      i++;
      y += 20;
   }
}

file_dialog::file_dialog(int cmd) : managed_window("File Selector")
{
   char   path[PATH_MAX];

   Background(inside_3D);
   Resize(400, 360);

   OK = new command_button(*this, 20, 300, "OK", OPEN_DLG_OK);
   OK_command = cmd;

   Cancel = new command_button(*this, 300, 300, "Cancel", OPEN_DLG_CANCEL);
   Command_WhenClosed(OPEN_DLG_CANCEL);

   diredit = new edit(*this, 20, 20, "");
   diredit->Resize(360, 20);

   directory_list = new directory_view(*this);
   directory_list->Move(20, 44);
   fileedit = new edit(*this, 20, 270, "");
   fileedit->Resize(360, 20);

   getcwd(path, PATH_MAX);
   *diredit = String(path);
   directory_list->changedir(path);
}

int file_dialog::DoCommand(int code)
{
   switch (code)
   {
   case OPEN_DLG_OK:
      Unmap();
      XApp->DoCommand(OK_command);
      return 1;

   case OPEN_DLG_CANCEL:
      Unmap();
      return 1;

   default:
      return -1;
   }
}

void file_dialog::ChildMessage(win_message &msg)
{

   if (msg.msg_id == EDIT_ENTERED)
   {
      if (msg.from == fileedit)
      {
         diredit->SetFocus();
      }
      else
      {
         *diredit = directory_list->changedir(*diredit);
         fileedit->SetFocus();
      }
   }
   else if (msg.msg_id == TABLE_ROW_SELECT)
   {
      if (directory_list->selected_a_dir())
      {
         String newdir;

         newdir = directory_list->cell(directory_list->selected(), 1);
         *diredit = directory_list->changedir(newdir);
      }
      else
      {
         *fileedit = directory_list->cell(directory_list->selected(), 1);
      }
   }
}
