
/**************************************************************************
**  (c) Copyright 1998, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : frame.cpp
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.1 $
**
**  DESCRIPTION      :  Implementation of class frame
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Mar 07, 1998
**      LAST UPDATE     : Mar 07, 1998
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: frame.cpp,v $
   Revision 1.1  2002/07/25 08:01:27  arjen
   First checkin, AXE release 0.2

*****************************/

static const char *RCSID = "$Id: frame.cpp,v 1.1 2002/07/25 08:01:27 arjen Exp $";

#include "frame.h"

/*=========================================================================
**  NAME           : EV_Expose - Handle Expose event.
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

int  frame::EV_Expose(XExposeEvent ev)
{
   draw_effect();
   place_text();
   redraw();
   return 1;
}

/*=========================================================================
**  NAME           : draw_effect - Draw the border effects in a frame
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

void frame::draw_effect(void)
{
   gc  *top_gc, *bottom_gc;

   unsigned w, h;
   int      thick;

   if (strength > 0)
   {
      top_gc = &light_3D_gc;
      bottom_gc = &dark_3D_gc;
      thick = strength;
   }
   else
   {
      top_gc = &dark_3D_gc;
      bottom_gc = &light_3D_gc;
      thick = -strength;
   }

   Size(w, h);
   for (int i=0; i< thick; i++)
   {
      XDrawLine(stddpy, ID(), *top_gc, i, i, i, h-i);
      XDrawLine(stddpy, ID(), *top_gc, i, i, w-i, i);

      XDrawLine(stddpy, ID(), *bottom_gc, i, h-i, w-i, h-i);
      XDrawLine(stddpy, ID(), *bottom_gc, w-i, i, w-i, h-i);
   }

}


/*=========================================================================
**  NAME           : place_text - Place aligned text in a frame
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

void frame::place_text(void)
{
   int  x, y;
   unsigned w, h;

   if (text && text[0])
   {
      Size(w, h);
      switch (halign)
      {
      case Left:
         x = strength;
         break;

      case Center:
         x = (w - default_font.TextWidth(text)) / 2;
         break;

      case Right:
         x = w - default_font.TextWidth(text) - strength;
         break;

      }

      switch (valign)
      {
         int  space;

      case Top:
         y = default_font.ascent() + strength;
         break;

      case Middle:
         space = (h - (default_font.ascent() + default_font.descent())) / 2;
         y = space + default_font.ascent();
         break;

      case Bottom:
         y = h - default_font.descent() - strength;
         break;

      }

      XDrawString(stddpy, ID(), menu_normal_gc, x, y, text, strlen(text));
   }
}

