
/**************************************************************************
**  (c) Copyright 1998, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : frame.h
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.1 $
**
**  DESCRIPTION      :  Definition of the frame class
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Mar 06, 1998
**      LAST UPDATE     : Mar 13, 1998
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: frame.h,v $
   Revision 1.1  2002/07/25 08:01:27  arjen
   First checkin, AXE release 0.2

*****************************/

/* static const char *RCSID = "$Id: frame.h,v 1.1 2002/07/25 08:01:27 arjen Exp $"; */

#ifndef FRAME_H
#define FRAME_H

#include "xappl.h"


/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : frame - window with border effect and aligned text
//  BASECLASS      : window
//  MEMBERS        : text     : The text to draw in the frame (may be 0).
//                   valign   : Vertical alignment of the text
//                   halign   : Horizontal alignment of the text.
//                   effect   : Type of border: 3D, Shadow, etc
//                   strength : Of the border effect. 0 = no effect, 
//                              positive means 'Up'; negative is 'Down'
//
//  OPERATORS      :
//  METHODS        : PRIVATE draw_effect()  : Draw border effect
//                   PRIVATE place_text()   : Draw the aligned text
//
//                   TextAlign()  : Set aligment of the text
//                   Strength()   : Change or read the strength of the effect.
//
//  DESCRIPTION    : A frame adds two visual effects to a window:
//                     1.  A border effect (3D or shadow)
//                     2.  An aligned line of text.
//
//  RELATIONS      :
//  SEE ALSO       :
//  LAST MODIFIED  :
///////////////////////////////////////////////////////////////////////////
*/

enum   hor_alignment { Left, Center, Right };
enum   ver_alignment { Top, Middle, Bottom };

class frame : public window
{
   char      *text;

   hor_alignment halign;
   ver_alignment valign;

   enum      { TriD, Shadow } effect;

   int       strength;

   void draw_effect(void);

   void place_text(void);

   virtual int  EV_Expose(XExposeEvent ev);

public:

   frame(window &par, int x, int y, int s = 3) : window(par, x, y, 80, 30)
   {
      Background(inside_3D);
      strength = s;
      effect   = TriD;
      text     = 0;
      halign   = Center;
      valign   = Middle;
      SelectInput(ExposureMask, 1);
   }

   frame(window &par, int x, int y, char *t) : window(par, x, y, 80, 30)
   {
      Background(inside_3D);
      strength = 5;
      effect   = TriD;
      text     = t;
      halign   = Center;
      valign   = Middle;
      SelectInput(ExposureMask, 1);
   }

   void TextAlign(hor_alignment hor, ver_alignment ver)
   {
      halign = hor;
      valign = ver;
      place_text();
   }

   void Strength(int s)
   {
      strength = s;
      draw_effect();
   }
   int Strength(void)
   {
      return strength;
   }

   virtual void redraw(void)
   {
   }
};

#endif /* FRAME_H */

