/**************************************************************************
**  (c) Copyright 1998, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : gc.h
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.1 $
**
**  DESCRIPTION      : Definition of gc class 
**
**  EXPORTED OBJECTS : class gc
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Feb 22, 1998
**      LAST UPDATE     : Oct 28, 1999
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: gc.h,v $
   Revision 1.1  2002/07/25 08:01:27  arjen
   First checkin, AXE release 0.2

*****************************/

/* static const char *RCSID = "$Id: gc.h,v 1.1 2002/07/25 08:01:27 arjen Exp $"; */

#include <X11/Xlib.h>
#include "display.h"
#include "color.h"
#include "font.h"
#include "pattern.h"

/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : gc
//  BASECLASS      :
//  MEMBERS        : XID gc_id
//  OPERATORS      :
//  METHODS        :
//
//  DESCRIPTION    :
//
//  RELATIONS      :
//  SEE ALSO       :
//  LAST MODIFIED  : Oct 28, 1999
///////////////////////////////////////////////////////////////////////////
*/

class gc
{
   GC  gc_id;

public:

   gc()
   {
      gc_id = 0;
   }

   gc(const color &fg)
   {
      gc_id = XCreateGC(stddpy, stddpy.Root(), 0, 0);
      XSetForeground(stddpy, gc_id, fg);
   }

   gc(font &fn, color &fg)
   {
      gc_id = XCreateGC(stddpy, stddpy.Root(), 0, 0);
      if (fn)
      {
         XSetFont(stddpy, gc_id, fn);
      }
      XSetForeground(stddpy, gc_id, fg);
   }

   gc(font &fn, color &fg, stipple &stip)
   {
      gc_id = XCreateGC(stddpy, stddpy.Root(), 0, 0);
      if (fn)
      {
         XSetFont(stddpy, gc_id, fn);
      }
      XSetForeground(stddpy, gc_id, fg);
      XSetStipple(stddpy, gc_id, stip);
      XSetFillStyle(stddpy, gc_id, FillStippled);

   }

   ~gc()
   {
      XFreeGC(stddpy, gc_id);
   }

   operator GC() const
   {
      return gc_id;
   }

   void Create()
   {
      gc_id = XCreateGC(stddpy, stddpy.Root(), 0, 0);
   }

   void SetFunction(int func)
   {
      XSetFunction(stddpy, gc_id, func);
   }

   void SetForeground(unsigned long fg)
   {
      XSetForeground(stddpy, gc_id, fg);
   }

   void SetBackground(unsigned long fg)
   {
      XSetBackground(stddpy, gc_id, fg);
   }

   void SetFont(font &fn)
   {
      if (fn)
      {
         XSetFont(stddpy, gc_id, fn);
      }
   }

   void SetLineAttributes(unsigned width, int line_style = LineSolid,
                           int cap_style = CapButt, int join_style = JoinMiter)
   {
      XSetLineAttributes(stddpy, gc_id, width, line_style,
                                    cap_style, join_style);
   }
};

