/**************************************************************************
**  (c) Copyright 1999, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : integer.h
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.2 $
**
**  DESCRIPTION      :  Arbitrary length integer
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Feb 06, 1998
**      LAST UPDATE     : Oct 16, 1999
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: integer.h,v $
   Revision 1.2  2002/11/04 07:24:31  arjen
   Use proper namespace for iostream classes

   Revision 1.1  2002/07/25 08:01:27  arjen
   First checkin, AXE release 0.2

*****************************/

/* static const char *RCSID = "$Id: integer.h,v 1.2 2002/11/04 07:24:31 arjen Exp $"; */


#include <iostream>

/*
 *  The integer class (not int !) implements an arbitrary length
 *  number. It allocates more words as the number grows.
 *  When a number consists of more than one word, only the first
 *  is signed (the most significant), the rest of the words is
 *  unsigned.
 *
 *  When adding, overflow is detected if the addends are of the same
 *  sign and the result has a different sign.
 */

class integer
{
   int nr_words;
   short msw;     // Most significant word, in 2s complement
   unsigned short *Number;  // least significant words.

public:

   integer()
   {
      nr_words = 0;
      msw = 0;
   }

   integer (long l)
   {
      nr_words = 1;
      Number = new unsigned short[1];
      Number[0] = l;
      msw = l >> 16;
   }

   ~integer()
   {
      if (nr_words > 0)
         delete Number;
   }

   integer& operator=(integer &);
   integer operator+(integer &);
//   operator-()
//   operator*()
//   operator/()
//   operator%()
//   operator<<()
//   operator>>()

   friend std::ostream& operator<<(std::ostream&, integer&);
   friend std::istream& operator>>(std::istream&, integer&);
};
