/**************************************************************************
**  (c) Copyright 1998, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : menu.cpp
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.1 $
**
**  DESCRIPTION      : Implementation of menu classes 
**
**  EXPORTED OBJECTS : menu_container
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Feb 13, 1998
**      LAST UPDATE     : Mar 07, 1998
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: menu.cpp,v $
   Revision 1.1  2002/07/25 08:01:27  arjen
   First checkin, AXE release 0.2

*****************************/

static const char *RCSID = "$Id: menu.cpp,v 1.1 2002/07/25 08:01:27 arjen Exp $";

#include "menu.h"

#include <stdio.h>


/*=========================================================================
**  NAME           : RearrangeItems
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

void menu_container::RearrangeItems(unsigned &ov_width, unsigned &ov_height)
{
   win_list *l;
   unsigned width, height;

   ov_width = 0;
   ov_height = 0;

   if (mode == Horizontal)
   {
      for(l = children; l; l = l->next)
      {
         l->child->Move(ov_width, 0);
         l->child->Size(width, height);
         ov_width += width;
         ov_height = height > ov_height ? height : ov_height;
      }

      for(l = children; l; l = l->next)
      {
         l->child->Size(width, height);
         l->child->Resize(width, ov_height);
      }
   }
   else   /*  mode is Vertical */
   {
      for(l = children; l; l = l->next)
      {
         l->child->Move(0, ov_height);
         l->child->Size(width, height);
         ov_height += height;
         ov_width = width > ov_width ? width : ov_width;
      }

      for(l = children; l; l = l->next)
      {
         l->child->Size(width, height);
         l->child->Resize(ov_width, height);
      }
   }

}

/*=========================================================================
**  NAME           : HideSubmenus
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

void menu_container::HideSubmenus(void)
{
   //printf("HideSubmenus(0x%x)\n",this);

   win_list *l = children;

   while (l)
   {
      menu_item *mi;

      mi = (menu_item *)l->child;
      mi->HideSubmenu();

      l = l->next;
   }

}

/*=========================================================================
**  NAME           : Show
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

void menu_container::Show(menu_item *by, int x, int y)
{
   shown_by = by;
   Move(x, y);
   Realize();
}


void menu_item::redraw(void)
{
}

/*=========================================================================
**  NAME           : EV_EnterNotify
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

int menu_item::EV_EnterNotify(XCrossingEvent ev)
{
   highlighted = 1;
   Background(menu_highlight);
   Clear();

   redraw();

   GrabPointer(hand_cursor);

   //  Make sure all sibling popup menus are disappeared.

   win_list *l = parent->children;

   while (l)
   {
      menu_item *mi;

      mi = (menu_item *)l->child;
      if (mi != this)
         mi->HideSubmenu();

      l = l->next;
   }

   ShowSubmenu(ev.x_root - ev.x, ev.y_root - ev.y);

   return 1;
}

/*=========================================================================
**  NAME           : EV_LeaveNotify
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

int menu_item::EV_LeaveNotify(XCrossingEvent ev)
{
   highlighted = 0;
   Background(menu_normal);
   Clear();
   redraw();

   return 1;
}

/*=========================================================================
**  NAME           : EV_ButtonRelease
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

int menu_item::EV_ButtonRelease(XButtonEvent ev)
{
      menu_container   *contain;

      UngrabPointer();

      contain = (menu_container *)parent;
      while (contain->shown_by)
      {
         menu_item *mi = (menu_item *)contain->shown_by;
         contain = (menu_container *)mi->parent;
      }
      //printf("contain->HideSubmenus(0x%x)\n", contain);
      contain->HideSubmenus();

      if (highlighted && !popup)
      {
         return XApp->SendCommand(command_code);
      }

      return 1;

}

/*=========================================================================
**  NAME           : HideSubmenu
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

void menu_item::HideSubmenu(void)
{
   if (popup)
   {
      //printf("popup->HideSubmenus(0x%x)\n", popup);
      popup->HideSubmenus();
      popup->Unmap();
      popup->shown_by = 0;
   }
}

/*=========================================================================
**  NAME           : ShowSubmenu
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

void menu_item::ShowSubmenu(int x, int y)
{
   if (popup)
   {
      unsigned w, h;

      menu_container *contain;

      Size(w, h);

      contain = (menu_container *)parent;
      contain->PositionSubmenu(x, y, w, h);

      popup->Show(this, x, y);
   }

}


/*=========================================================================
**  NAME           : redraw - Draw a label in a menu item
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Mar 07, 1998
**=========================================================================
*/

void menu_label::redraw(void)
{
   XDrawString(stddpy, ID(), enabled ? menu_normal_gc : menu_dimmed_gc,
                 5, default_font.ascent() + 3, label, strlen(label));

}

/*=========================================================================
**  NAME           : redraw -- Draw a separator in a menu item
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

void menu_separator::redraw(void)
{
   unsigned int  w, h;

   Size(w, h);
   XDrawLine(stddpy, ID(), menu_normal_gc, 0, 3, w, 3);
}
