/**************************************************************************
**  (c) Copyright 1998, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : pixmap.h
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.2 $
**
**  DESCRIPTION      : Definition of pixmap classes: pixmap, bitmap
**
**  EXPORTED OBJECTS : class pixmap
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Jul 13, 2000
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: pixmap.h,v $
   Revision 1.2  2002/11/04 07:24:31  arjen
   Use proper namespace for iostream classes

   Revision 1.1  2002/07/25 08:01:27  arjen
   First checkin, AXE release 0.2

*****************************/

/* static const char *RCSID = "$Id: pixmap.h,v 1.2 2002/11/04 07:24:31 arjen Exp $"; */

#ifndef AXE_PIXMAP_H
#define AXE_PIXMAP_H

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include "display.h"

/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : pixmap
//  BASECLASS      :
//  MEMBERS        : Pixmap pixm;
//  OPERATORS      :
//  METHODS        :
//
//  DESCRIPTION    :
//
//  RELATIONS      :
//  SEE ALSO       :
//  LAST MODIFIED  : Jul 13, 2000
///////////////////////////////////////////////////////////////////////////
*/

class pixmap
{
   Pixmap pixm;
   unsigned   w, h;   //  Size of the pixmap

public:

   pixmap()
   {
      pixm = 0;
   }

   pixmap(char *data, unsigned int w, unsigned int h)
   {
      pixm = XCreateBitmapFromData(stddpy, stddpy.Root(), data, w, h);
   }

   pixmap(char *filename)
   {
      XpmAttributes    attr;

      attr.valuemask = 0;

      XpmReadFileToPixmap(stddpy, stddpy.Root(), filename, &pixm, NULL, &attr);
      w = attr.width;
      h = attr.height;
   }

   pixmap(char *data[])
   {
      XpmAttributes    attr;

      attr.valuemask = 0;

      XpmCreatePixmapFromData(stddpy, stddpy.Root(), data, &pixm, NULL, &attr);
      w = attr.width;
      h = attr.height;
   }

   ~pixmap()
   {
      if (pixm != 0)
      {
         XFreePixmap(stddpy, pixm);
      }
   }

   void read(char *filename)
   {
extern pixmap nopicture;
      XpmAttributes    attr;
      int              error;

      if (pixm != 0)
      {
         XFreePixmap(stddpy, pixm);
      }

      attr.valuemask = 0;
      error = XpmReadFileToPixmap(stddpy, stddpy.Root(), filename, &pixm, NULL, &attr);
      if (error != XpmSuccess)
      {
         std::cerr << "Error " << error << " reading XPM file " << filename << "\n";
         pixm = nopicture.pixm;
         w = nopicture.w;
         h = nopicture.h;
      }
      else
      {
         w = attr.width;
         h = attr.height;
      }
   }

   void create(char *data[])
   {
      XpmAttributes    attr;

      if (pixm != 0)
      {
         XFreePixmap(stddpy, pixm);
      }

      attr.valuemask = 0;

      XpmCreatePixmapFromData(stddpy, stddpy.Root(), data, &pixm, NULL, &attr);
      w = attr.width;
      h = attr.height;
   }

   operator Pixmap() const
   {
      return pixm;
   }

   size Size() const
   {
      return size(w, h);
   }
};


#endif /* AXE_PIXMAP_H  */
