
/**************************************************************************
**  (c) Copyright 1998, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : scroll.cpp
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.1 $
**
**  DESCRIPTION      :  Implementation of scrollbar class.
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Mar 20, 1998
**      LAST UPDATE     : Apr 04, 1998
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: scroll.cpp,v $
   Revision 1.1  2002/07/25 08:01:27  arjen
   First checkin, AXE release 0.2

*****************************/

static const char *RCSID = "$Id: scroll.cpp,v 1.1 2002/07/25 08:01:27 arjen Exp $";

#include "scroll.h"

slider::slider(scrollbar *b) : frame(*b, 0, 0, 2)
{
   bar = b;
   slide_start_y = 7;
   Resize(14,14);
   SelectInput(ButtonPressMask|Button1MotionMask|ButtonReleaseMask, 1);
   Map();
}

/*=========================================================================
**  NAME           : EV_MotionNotify
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : Move the mouse in the slider with button 1 pressed.
**                   move_slider() cannot be used directly because the
**                   motion is in physical coordinates and move_slider()
**                   needs an offset in logical coordinates. The function
**                   scrollbar::map_mouse() performs the translation.
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Apr 04, 1998
**=========================================================================
*/

int slider::EV_MotionNotify(XMotionEvent ev)
{
   bar->move_slider(bar->map_mouse(ev.y - slide_start_y));

   return 1;
}

/*=========================================================================
**  NAME           : move_slider
**  SYNOPSIS       : void scrollbar::move_slider(int offset)
**  PARAMETERS     : offset : change in the slider's position
**  RETURN VALUE   : None
**
**  DESCRIPTION    : Moves the slider within the scrollbar. Called when
**                   the mouse is clicked in the scrollbar or when the
**                   slider is dragged.
**                   The offset is in logical coordinates.
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Apr 04, 1998
**=========================================================================
*/

void scrollbar::move_slider(int offset)
{
   int    physical_position;
   size   scroll_size;

   position += offset;

   //  Clip the slider in its range.

   if (position < 0)
      position = 0;
   if (position + slider_size > bar_size)
      position = bar_size - slider_size;

   scroll_size = Size();
   physical_position = position * scroll_size.h / bar_size;

   sl->Move(0, physical_position);
   ParentMessage(SCROLL_POSITIONED);
}

/*=========================================================================
**  NAME           : SetSizes
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Aug 30, 1999
**=========================================================================
*/

void scrollbar::SetSizes(unsigned s_bar, unsigned s_slider)
{
   size scroll_size;
   size physical_slider;

   //  Needs to recalculate the position in logical coordinates.

   bar_size = s_bar;
   slider_size = s_slider;

   scroll_size = Size();

   physical_slider.w = 14;
   physical_slider.h = slider_size * scroll_size.h / bar_size;
   if (physical_slider.h < 5)
      physical_slider.h = 5;

   sl->Resize(physical_slider);
}

