
/**************************************************************************
**  (c) Copyright 1998, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : scroll.h
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.1 $
**
**  DESCRIPTION      :  Definition of scrollbar class.
**                      A scrollbar is a construction with two frames. The
**                      scrollbar itself and a slider that moves within
**                      the scrollbar.
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Mar 20, 1998
**      LAST UPDATE     : Oct 09, 1999
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: scroll.h,v $
   Revision 1.1  2002/07/25 08:01:27  arjen
   First checkin, AXE release 0.2

*****************************/

/* static const char *RCSID = "$Id: scroll.h,v 1.1 2002/07/25 08:01:27 arjen Exp $"; */

#ifndef _SCROLL_H
#define _SCROLL_H

#include "frame.h"

class scrollbar;

/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : slider
//  BASECLASS      : frame
//  MEMBERS        :
//  OPERATORS      :
//  METHODS        :
//
//  DESCRIPTION    :
//
//  RELATIONS      :
//  SEE ALSO       :
//  LAST MODIFIED  : Oct 09, 1999
///////////////////////////////////////////////////////////////////////////
*/

class slider : public frame
{
   scrollbar  *bar;

   int        slide_start_y;

public:

   slider(scrollbar *b);
   
   virtual int EV_ButtonPress(XButtonEvent ev)
   {
      slide_start_y = ev.y;

      return 1;
   }

   virtual int EV_ButtonRelease(XButtonEvent ev)
   {
      return 1;
   }

   virtual int EV_MotionNotify(XMotionEvent ev);
};

/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : scrollbar
//  BASECLASS      : frame
//  MEMBERS        :
//  OPERATORS      :
//  METHODS        :
//
//  DESCRIPTION    :
//                   The bar_size, slider_size and position are defined
//                   in logical coordinates.
//                   The bar_size represents the size of the data being
//                   scrolled and corresponds to the total size of the
//                   scrollbar. The slider_size and position are mapped to
//                   physical coordinates using the ratio of the bar_size.
//
//                   physical coord = logical coord * window size / bar size.
//
//  RELATIONS      :
//  SEE ALSO       :
//  LAST MODIFIED  : Apr 03, 1998
///////////////////////////////////////////////////////////////////////////
*/

class scrollbar : public frame
{
   slider  *sl;

   unsigned   bar_size, slider_size;

   int     position;

public:

   scrollbar(window &par, unsigned size, int x, int y)
      : frame(par, x, y, -2)
   {
      Resize(16, size);
      bar_size = size;
      slider_size = 16;
      sl = new slider(this);
      position = 0;
      SelectInput(ButtonPressMask, 1);
   }
   
   ~scrollbar()
   {
      delete sl;
   }

   //  Map physical pixel coordinate to logical coordinate.

   int map_mouse(int mouse)
   {
      size scroll_size = Size();
      int  physical;

      physical =  mouse * (int)bar_size / scroll_size.h;

      return  physical;
   }

   void move_slider(int offset);

   virtual int EV_ButtonPress(XButtonEvent ev)
   {
      if (map_mouse(ev.y) > position)
         move_slider(slider_size);
      else
         move_slider(-slider_size);

      return 1;
   }

   operator int()
   {
      return position;
   }

   void SetSizes(unsigned s_bar, unsigned s_slider);
};

#endif /* _SCROLL_H */
