/**************************************************************************
**  (c) Copyright 1999, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : table.cpp
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.1 $
**
**  DESCRIPTION      :  Implementation of table_view class
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Aug 26, 1999
**      LAST UPDATE     : Aug 26, 1999
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: table.cpp,v $
   Revision 1.1  2002/07/25 08:01:27  arjen
   First checkin, AXE release 0.2

*****************************/

static const char *RCSID = "$Id: table.cpp,v 1.1 2002/07/25 08:01:27 arjen Exp $";

#include "table.h"

void table_view::redraw(void)
{
   int      i, j, x, y;
   unsigned w, h;
   unsigned desired_height;
   unsigned font_height = 20;  //  For the time being...

   Size(w, h);
   g.SetLineAttributes(2);
   XDrawLine(stddpy, ID(), g, 0, 20, w, 20);

   //  Determine if we need the scrollbar

   desired_height = (nr_rows + 1) * font_height;
   if (desired_height > h)
   {
      unsigned visible_rows;

      visible_rows = (h - font_height) / font_height;
      Vscroll->SetSizes(nr_rows, visible_rows);
      Vscroll->Move(w-18, font_height+1);
      Vscroll->Resize(16, h - font_height - 2);
      Vscroll->Map();
   }
   else
   {
      //  All rows fit in the window

      Vscroll->Unmap();
   }

   //  Draw the grid

   g.SetLineAttributes(0);
   x = 0;
   for (i = 0; i < nr_columns; i++)
   {
      x += Head[i].width;
      XDrawLine(stddpy, ID(), g, x, 0, x, h);
   }

   y = font_height;
   g.SetLineAttributes(0, LineOnOffDash);
   XSetDashes(stddpy, g, 0, "\001\001", 2);
   for (j = 0; j < nr_rows && (unsigned)y < h; j++)
   {
      y += font_height;
      XDrawLine(stddpy, ID(), g, 0, y, w, y);
   }

   //  Draw the header

   g.SetFont(header_font);
   x = 2;                    // small margin
   for (i = 0; i < nr_columns; i++)
   {
      if ((char *)(Head[i].title))
      {
         XDrawString(stddpy, ID(), g, x, 15, Head[i].title, ~Head[i].title);
      }
      x += Head[i].width;
   }

   //  Draw the cells, depending on the scroll position.

   g.SetFont(cell_font);
   y = 35;
   for (i = *Vscroll; i < nr_rows && (unsigned)y < h; i++)
   {
      x = 2;
      for (j = 0; j < nr_columns; j++)
      {
         XClearArea(stddpy, ID(), x, y - 14, Head[j].width - 2,
                                          font_height -2, False);
         if (i == selected_row)
         {
            XFillRectangle(stddpy, ID(), g, x, y - 14, Head[j].width - 2,
                                          font_height -2);
            g.SetForeground(stddpy.White());
         }
         if ((char *)(Cells[i][j]))
         {
            int  text_x, text_width;

            text_width = cell_font.TextWidth(Cells[i][j]);
            switch (Head[j].alignment)
            {
            case Left:
               text_x = x;
               break;
            case Center:
               text_x = x + (Head[j].width - text_width) / 2;
               break;
            case Right:
               text_x = x + Head[j].width - text_width - 4;
               break;
            }
            XDrawString(stddpy, ID(), g, text_x, y, Cells[i][j], ~Cells[i][j]);
         }
         g.SetForeground(stddpy.Black());
         x += Head[j].width;
      }
      y += font_height;
   }
}

int table_view::EV_ButtonPress(XButtonEvent ev)
{
   unsigned font_height = 20;  //  Should be obtained from cell_font
   int      selected;

   if ((unsigned)ev.y > font_height)
   {
      //   Below the header

      selected = (ev.y - font_height) / font_height + *Vscroll;
      if (selected < nr_rows)
      {
         selected_row = selected;
         redraw();
         ParentMessage(TABLE_ROW_SELECT);
      }
   }
   return 1;
}

void table_view::ChildMessage(win_message &msg)
{
   if (msg.msg_id == SCROLL_POSITIONED)
   {
      redraw();
   }
}

