/**************************************************************************
**  (c) Copyright 1999, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : table.h
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.1 $
**
**  DESCRIPTION      : Definition of table_view class 
**
**  EXPORTED OBJECTS : class table_view
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Aug 26, 1999
**      LAST UPDATE     : Aug 26, 1999
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: table.h,v $
   Revision 1.1  2002/07/25 08:01:27  arjen
   First checkin, AXE release 0.2

*****************************/

/* static const char *RCSID = "$Id: table.h,v 1.1 2002/07/25 08:01:27 arjen Exp $"; */

#include "String.h"
#include "scroll.h"

/*
///////////////////////////////////////////////////////////////////////////
//  NAME           : table_view
//  BASECLASS      : window
//  MEMBERS        : 
//  OPERATORS      :
//  METHODS        :
//
//  DESCRIPTION    :
//
//  RELATIONS      :
//  SEE ALSO       :
//  LAST MODIFIED  : Aug 26, 1999
///////////////////////////////////////////////////////////////////////////
*/

class table_view : public window
{

   struct header
   {
      String         title;
      hor_alignment  alignment;
      int            width;
   };

   header   *Head;
   String   **Cells;

   int nr_rows, nr_columns;

   int selected_row;

   scrollbar  *Vscroll;
   font       header_font, cell_font;
   gc         g;

public:

   table_view(window &parent, int rows, int columns)
       : window(parent)
   {
      int i;

      Vscroll = new scrollbar(*this, 120, 260, 16);

      Head  = new header[columns];
      Cells = new String * [rows];
      for (i = 0; i < rows; i++)
      {
         Cells[i] = new String[columns];
      }

      for (i = 0; i < columns; i++)
      {
         Head[i].alignment = Left;
         Head[i].width     = 80;
      }
      nr_rows     = rows;
      nr_columns  = columns;
      selected_row = -1;

      g.Create();
      g.SetForeground(Black);
      g.SetBackground(stddpy.White());
      header_font.Load("6x13bold");
      cell_font.Load("6x13");
      SelectInput(ExposureMask | ButtonPressMask, 1);
   }

   ~table_view()
   {
      delete Vscroll;
      delete [] Head;
      for (int i = 0; i < nr_rows; i++)
      {
         delete [] Cells[i];
      }
      delete [] Cells;
   }

   void recreate(int rows)
   {
      int    i;

      for (i = 0; i < nr_rows; i++)
      {
         delete [] Cells[i];
      }
      delete [] Cells;

      nr_rows = rows;
      Cells = new String * [nr_rows];
      for (i = 0; i < nr_rows; i++)
      {
         Cells[i] = new String[nr_columns];
      }
      selected_row = -1;
      Vscroll->move_slider(-*Vscroll);
      Clear();
      redraw();
   }

   //  Provide access to the cells of the table

   String & head(int col)
   {
      return Head[col].title;
   }

   String & cell(int row, int col)
   {
      return Cells[row][col];
   }

   int selected(void)
   {
      return selected_row;
   }

   void top_row(int row_nr)
   {
      Vscroll->move_slider(row_nr - *Vscroll);
   }

   int top_row()
   {
      return int(*Vscroll);
   }

   //  Functions for layout control

   void align(int col, hor_alignment al)
   {
      Head[col].alignment = al;
   }

   int & width(int col)
   {
      return Head[col].width;
   }

   virtual void redraw();

   virtual int EV_Expose(XExposeEvent ev)
   {
      redraw();
      return 1;
   }

   virtual int EV_ButtonPress(XButtonEvent);

   virtual void ChildMessage(win_message &msg);
};

