/**************************************************************************
**  (c) Copyright 1998, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : xappl.cpp
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.1 $
**
**  DESCRIPTION      : Implementation of xapplication class 
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Feb 06, 1998
**      LAST UPDATE     : Feb 22, 1998
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: xappl.cpp,v $
   Revision 1.1  2002/07/25 08:01:27  arjen
   First checkin, AXE release 0.2

*****************************/

static const char *RCSID = "$Id: xappl.cpp,v 1.1 2002/07/25 08:01:27 arjen Exp $";

#include "xappl.h"

#include <stdio.h>

static char dim_bits[] =
{
   0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA
};

#include "nopicture.xpm"

xapplication *XApp;
display     stddpy;

color       menu_normal("light grey"), menu_highlight("slate grey");
color       inside_3D(0xB000, 0xC000, 0xD000),
            light_rim_3D(0xEF00, 0xF000, 0xFFFF),
            dark_rim_3D(0x7000, 0x8000, 0x9000);
color       Black("black"), White("white");
color       _Red("red"), _Green("green");

font        default_font("variable");
font        fixed("fixed");

cursor      hand_cursor(XC_hand1);
cursor      text_cursor(XC_xterm);
cursor      busy_cursor(XC_watch);

stipple     dim_pattern(dim_bits, 8, 8);

gc          menu_normal_gc(default_font, Black);
gc          menu_dimmed_gc(default_font, Black, dim_pattern);
gc          inside_3D_gc(inside_3D),
            light_3D_gc(light_rim_3D), dark_3D_gc(dark_rim_3D);

gc          red_gc(default_font, _Red), green_gc(default_font, _Green);
gc          white_gc(White);

gc          text_normal_gc(fixed, Black);
gc          edit_cursor_gc(fixed, Black);

pixmap      nopicture(nopicture_xpm);

xapplication::xapplication()
{
   XApp = this;
   Pgrab_window = None;

}

void xapplication::add_window(XID w_id, window *w_ptr)
{
   wintab.xh_add(w_id, w_ptr);
}

void xapplication::remove_window(XID w_id)
{
   wintab.xh_del(w_id);
}

void xapplication::Pgrab(window *w_ptr)
{
   Pgrab_window = w_ptr;
}

void xapplication::Topwin(managed_window *w_ptr)
{
   Top_window = w_ptr;
}

/*=========================================================================
**  NAME           : 
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

void xapplication::Initialize(void)
{
   XSetFont(stddpy, stddpy.gc(), default_font);
   edit_cursor_gc.SetFunction(GXxor);
   edit_cursor_gc.SetForeground(stddpy.Black() ^ stddpy.White());
}

/*=========================================================================
**  NAME           : 
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

void xapplication::ParseArguments(int argc, char *argv[])
{
}

void xapplication::SetupResources(void)
{
}

void xapplication::UserInit(void)
{
}

void xapplication::Cleanup(void)
{
}

int xapplication::DoCommand(int code)
{
   return 1;
}

int  xapplication::DispatchEvent(XEvent ev)
{
   window   *target_win;
   int      go_on;

   target_win = wintab[ev.xany.window];

   switch (ev.type)
   {
   case Expose:
      go_on = target_win->EV_Expose(ev.xexpose);
      break;

   case NoExpose:
      go_on = target_win->EV_NoExpose(ev.xnoexpose);
      break;

   case KeyPress:
      go_on = target_win->EV_KeyPress(ev.xkey);
      break;

   case ButtonPress:
      if (Pgrab_window == None || target_win->SelectInput() & GrabPointerMask)
         go_on = target_win->EV_ButtonPress(ev.xbutton);
      else
         go_on = Pgrab_window->EV_ButtonPress(ev.xbutton);
      break;

   case ButtonRelease:
      if (Pgrab_window == None || target_win->SelectInput() & GrabPointerMask)
         go_on = target_win->EV_ButtonRelease(ev.xbutton);
      else
         go_on = Pgrab_window->EV_ButtonRelease(ev.xbutton);
      break;

   case MotionNotify:
      go_on = target_win->EV_MotionNotify(ev.xmotion);
      break;

   case EnterNotify:
      go_on = target_win->EV_EnterNotify(ev.xcrossing);
      break;

   case LeaveNotify:
      go_on = target_win->EV_LeaveNotify(ev.xcrossing);
      break;

   case FocusOut:
      go_on = target_win->EV_FocusOut(ev.xfocus);
      break;

   case FocusIn:
      go_on = target_win->EV_FocusIn(ev.xfocus);
      break;

   case ConfigureNotify:
      go_on = target_win->EV_ConfigureNotify(ev.xconfigure);
      break;

   case ClientMessage:
      go_on = target_win->EV_ClientMessage(ev.xclient);
      break;

   default:
      go_on = target_win->EV_Default(ev);
      break;
   }

   return go_on;
}


int xapplication::SendCommand(int code)
{
   int  result;

   result = Top_window->DoCommand(code);
   if (result < 0)
   {
      result = DoCommand(code);
   }

   return result;
}

/*=========================================================================
**  NAME           : 
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

int main(int argc, char *argv[])
{
   XEvent ev;

   XApp->Initialize();
   XApp->ParseArguments(argc, argv);
   XApp->SetupResources();
   XApp->UserInit();

   do
   {
      stddpy >> ev;
   }
   while (XApp->DispatchEvent(ev));
};


