/**************************************************************************
**  (c) Copyright 1998, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : xwindow.cpp
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.2 $
**
**  DESCRIPTION      :  Implementation of the window class
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Feb 06, 1998
**      LAST UPDATE     : Feb 09, 1998
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: xwindow.cpp,v $
   Revision 1.2  2002/09/26 14:50:54  arjen
   Proper casting of event data in EV_ClientMessage()

   Revision 1.1  2002/07/25 08:01:27  arjen
   First checkin, AXE release 0.2

*****************************/

static const char *RCSID = "$Id: xwindow.cpp,v 1.2 2002/09/26 14:50:54 arjen Exp $";

#include "xappl.h"   /*  Includes xwindow.h   */

#include <stdio.h>


/*=========================================================================
**  NAME           : 
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

window::window(int x, int y, unsigned int _w, unsigned int _h, int bw)
{
   w = _w;
   h = _h;

   parent = 0;
   children = 0;

   win_id = XCreateSimpleWindow(stddpy.Dpy(), stddpy.Root(),
                    x, y, w, h, bw,
                   stddpy.Black(), stddpy.White());

   selection_mask = 0;

   XApp->add_window(win_id, this);
}


/*=========================================================================
**  NAME           : 
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

window::window(window &parentw, int x, int y,
               unsigned int _w, unsigned int _h, int bw)
{
   w = _w;
   h = _h;

   parent = &parentw;
   children = 0;
   parentw.add_child(this);

   win_id = XCreateSimpleWindow(stddpy.Dpy(), parentw.win_id,
                    x, y, w, h, bw,
                   stddpy.Black(), stddpy.White());

   selection_mask = 0;

   XApp->add_window(win_id, this);
}


/*=========================================================================
**  NAME           : 
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

window::~window()
{
   if (parent)
   {
      parent->remove_child(this);
   }

   XDestroyWindow(stddpy.Dpy(), win_id);

   XApp->remove_window(win_id);
}



/*=========================================================================
**  NAME           : add_child - Add subwindow in the list
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

void window::add_child(window *ch)
{
   win_list *l;

   l = new win_list;

   l->next = children;
   l->child = ch;
   children = l;
}

/*=========================================================================
**  NAME           : remove_child - Remove subwindow from the list
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

void window::remove_child(window *ch)
{
   win_list *l = 0, *prev = 0;

   l = children;
   while (l && l->child != ch)
   {
      prev = l;
      l = l->next;
   }

   // If l would be NULL, the child window would not be in
   // it's parent's list of children. This can not happen !

   if (prev)
   {
      prev->next = l->next;
   }
   else
   {
      children = l->next;
   }

   delete l;
}

/*=========================================================================
**  NAME           : EV_Expose
**  SYNOPSIS       : int window::EV_Expose(XExposeEvent ev)
**  PARAMETERS     :
**  RETURN VALUE   : 0 if the application should quit
**
**  DESCRIPTION    : Event handling functions.
**                   Most of the event handling functions are empty.
**                   Derived classes of window should override these functions
**                   to implement speecific behavior.
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

int window::EV_Expose(XExposeEvent ev)
{
   return 1;
}

int window::EV_NoExpose(XNoExposeEvent ev)
{
   return 1;
}

int window::EV_KeyPress(XKeyEvent ev)
{
   return 1;
}

int window::EV_KeyRelease(XKeyEvent ev)
{
   return 1;
}

int window::EV_ButtonPress(XButtonEvent ev)
{
   return 1;
}

int window::EV_ButtonRelease(XButtonEvent ev)
{
   return 1;
}

int window::EV_MotionNotify(XMotionEvent ev)
{
   return 1;
}

int window::EV_EnterNotify(XCrossingEvent ev)
{
   return 1;
}

int window::EV_LeaveNotify(XCrossingEvent ev)
{
   return 1;
}

int window::EV_FocusIn(XFocusChangeEvent ev)
{
   return 1;
}

int window::EV_FocusOut(XFocusChangeEvent ev)
{
   return 1;
}


/*=========================================================================
**  NAME           : EV_ConfigureNotify - Handle Configure event
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : Handles the event where the window is moved relative
**                   to its parent, resized or restacked.
**                   The default handler calls ParentResized for all of its
**                   children. The subwindows usually react by fitting their
**                   size to the size of the parent.
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Feb 10, 1998
**=========================================================================
*/

int window::EV_ConfigureNotify(XConfigureEvent ev)
{
   win_list *l;

   w = ev.width;
   h = ev.height;

   for (l = children; l; l = l->next)
   {
      l->child->ParentResized(ev.width, ev.height);
   }
   return 1;
}

int window::EV_ClientMessage(XClientMessageEvent ev)
{
   return 1;
}

int window::ParentResized(int w, int h)
{
   // stub

   return 1;
}

/* Event names.  Used in "type" field in XEvent structures.  Not to be
confused with event masks above.  They start from 2 because 0 and 1
are reserved in the protocol for errors and replies. */

static char * event_names[] =
{
   " Undefined(0)",
   " Undefined(1)",
   " KeyPress",
   " KeyRelease",
   " ButtonPress",
   " ButtonRelease",
   " MotionNotify",
   " EnterNotify",
   " LeaveNotify",
   " FocusIn",
   " FocusOut",
   " KeymapNotify",
   " Expose",
   " GraphicsExpose",
   " NoExpose",
   " VisibilityNotify",
   " CreateNotify",
   " DestroyNotify",
   " UnmapNotify",
   " MapNotify",
   " MapRequest",
   " ReparentNotify",
   " ConfigureNotify",
   " ConfigureRequest",
   " GravityNotify",
   " ResizeRequest",
   " CirculateNotify",
   " CirculateRequest",
   " PropertyNotify",
   " SelectionClear",
   " SelectionRequest",
   " SelectionNotify",
   " ColormapNotify",
   " ClientMessage",
   " MappingNotify"
};

int window::EV_Default(XEvent ev)
{
   return 1;
}

void window::ChildMessage(win_message &msg)
{
}

void window::ParentMessage(int id)
{
   win_message  msg;


   msg.msg_id = id;
   msg.from   = this;

   parent->ChildMessage(msg);
}

void window::Resize(unsigned width, unsigned height)
{
   w  = width;
   h  = height;

   XResizeWindow(stddpy.Dpy(), win_id, w, h);
}

void window::Resize(size sz)
{
   w  = sz.w;
   h  = sz.h;

   XResizeWindow(stddpy.Dpy(), win_id, w, h);
}

void window::Move(int x, int y)
{
   XMoveWindow(stddpy, win_id, x, y);
}

/*=========================================================================
**  NAME           : Background
**  SYNOPSIS       : void window::Background(unsigned long pixel)
**  PARAMETERS     :
**  RETURN VALUE   :
**
**  DESCRIPTION    : Set the pixel value of the window's background color
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

void window::Background(unsigned long pixel)
{
   XSetWindowAttributes attrib;

   attrib.background_pixel = pixel;
   XChangeWindowAttributes(stddpy.Dpy(), win_id, CWBackPixel, &attrib);
}

void window::WindowGravity(int gravity)
{
   XSetWindowAttributes attrib;

   attrib.win_gravity = gravity;
   XChangeWindowAttributes(stddpy.Dpy(), win_id, CWWinGravity, &attrib);
}

void window::Clear(void)
{
   XClearWindow(stddpy.Dpy(), win_id);
}

/*=========================================================================
**  NAME           : SelectInput - Turn events on or off.
**  SYNOPSIS       : long window::SelectInput(long events, int on)
**  PARAMETERS     : events   : Bitwise OR of event masks
**                   on       : 0 - Events off; 1 - Events on
**  RETURN VALUE   : The new event selection mask
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : 
**=========================================================================
*/

long window::SelectInput(long events, int on)
{
   if (on)
   {
      selection_mask |= events;
   }
   else
   {
      selection_mask &= ~events;
   }

   XSelectInput(stddpy.Dpy(), win_id, selection_mask);

   return selection_mask;
}

int  window::GrabPointer(Cursor curs)
{
   int status;

   status = XGrabPointer(stddpy, win_id, True,
          ButtonPressMask|ButtonReleaseMask|EnterWindowMask|LeaveWindowMask,
                GrabModeAsync, GrabModeAsync, None, curs, CurrentTime);
   XApp->Pgrab(this);

   return status;
}

int  window::UngrabPointer(void)
{
   int status;

   status = XUngrabPointer(stddpy, CurrentTime);
   XApp->Pgrab(None);

   return status;
}

int managed_window::EV_FocusIn(XFocusChangeEvent ev)
{
   XApp->Topwin(this);

   return 1;
}

int managed_window::EV_ClientMessage(XClientMessageEvent ev)
{
   if (ev.message_type == XInternAtom(stddpy, "WM_PROTOCOLS", True))
   {
      if ((Atom)ev.data.l[0] == XInternAtom(stddpy, "WM_DELETE_WINDOW", True))
      {
         return XApp->SendCommand(WM_Delete_Command);
      }
   }
   return 1;
}
